## File "EnhancedHeaderTool.py"
## -*- coding: utf-8 -*-
##########################################################################
#                                                                        #
#       written by: Pierre-Yves Landuré, pierre-yves@landure.org         #
#                                                                        #
##########################################################################

"""EnhancedHeaderTool's main class"""

# Python stuff
import logging
from Products.dcEnhancedHeader.config import PROJECTNAME

# Zope stuff
from Globals import InitializeClass 
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

# CMF stuff
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.CMFCorePermissions import View, ManagePortal
from Products.CMFCore.utils import UniqueObject, getToolByName

from Products.CMFPlone.PloneBaseTool import PloneBaseTool

from BTrees.OOBTree import OOBTree

# Time stuff.
from time import time

# CSV Stuff
import csv



def title_sorter(feed_a, feed_b):
    """ Dummy function to sort feed by names."""
    return cmp(feed_a['title'], feed_b['title'])



class EnhancedHeaderTool(PloneBaseTool, UniqueObject, SimpleItem, PropertyManager):
    """
    Provice tools and configuration properties for the enhanced header.
    """

    id = 'portal_enhancedheader'
    meta_type = 'Enhanced Header Tool'
    toolicon = 'skins/dcEnhancedHeader/enhancedheader_icon.gif'
    isPrincipiaFolderish = 0

    __implements__ = (PloneBaseTool.__implements__, SimpleItem.__implements__, )

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    manage_options = ( PropertyManager.manage_options
                     + SimpleItem.manage_options)

    def __init__(self, *args, **kwargs):
        """
        Enhanced header tool initialisation.
        """
        self._setProperty('dublin_core_metas', True, 'boolean')
        self._setProperty('rss_feeds_links', True, 'boolean')



    security.declareProtected(View, 'getDublinCoreMetas')
    def getDublinCoreMetas(self):
        """
        get the dublin_core_metas status.
        """

        try:
          # We get the configuration.
          return self.getProperty('dublin_core_metas', False);
        except Exception, e:
          # We can not get configuration. We fail silently.
          LOG.info("getDublinCoreMetas : Unable to get 'dublin_core_metas' property (Error : %(error)s)." % { 'error' : str(e), })
          return False



    security.declareProtected(View, 'getRssFeedsLinks')
    def getRssFeedsLinks(self):
        """
        get the rss_feeds_links status.
        """

        try:
          # We get the configuration.
          return self.getProperty('rss_feeds_links', False);
        except Exception, e:
          # We can not get configuration. We fail silently.
          LOG.info("getRssFeedsLinks : Unable to get 'rss_feeds_links' property (Error : %(error)s)." % { 'error' : str(e), })
          return False



    security.declareProtected(ManagePortal, 'setDublinCoreMetas')
    def setDublinCoreMetas(self, dublin_core_metas):
        """
        Set the dublin_core_metas status.
        """

        dublin_core_metas = bool(dublin_core_metas)

        self._updateProperty('dublin_core_metas', dublin_core_metas);



    security.declareProtected(ManagePortal, 'setRssFeedsLinks')
    def setRssFeedsLinks(self, rss_feeds_links):
        """
        Set the rss_feeds_links status.
        """

        rss_feeds_links = bool(rss_feeds_links)

        self._updateProperty('rss_feeds_links', rss_feeds_links);



    security.declareProtected(View, 'getMetaData')
    def getMetaData(self, context):
        """
        Get the MetaData of the given context.
        """

        LOG = logging.getLogger(PROJECTNAME)

        try:
          # We get the configuration.
          dublin_core_metas = self.getProperty('dublin_core_metas', False);
        except Exception, e:
          # We can not get configuration. We fail silently.
          dublin_core_metas = False
          LOG.info("getMetaData : Unable to get 'dublin_core_metas' property (Error : %(error)s)." % { 'error' : str(e), })


        if not dublin_core_metas:
            try:
                utils_tool = getToolByName(self, 'plone_utils')
            except AttributeError, e:
                LOG.error("getMetaData : Unable to get 'plone_utils' (Error : %(error)s)." % { 'error' : str(e), })
                return []

            return utils_tool.listMetaTags(context).items()
        else:
            hdrlist = []

            #   These two are import for most search engines
            hdrlist.append( ( 'description', context.Description() ) )
            hdrlist.append( ( 'keywords', ', '.join( context.Subject() ) ) )

            hdrlist.append( ( 'DC.description', context.Description() ) )
            hdrlist.append( ( 'DC.subject', ', '.join( context.Subject() ) ) )
            for creator in context.listCreators():
                hdrlist.append( ( 'DC.creator', creator ) )
            for contributor in context.listContributors():
                hdrlist.append( ( 'DC.contributor', contributor ) )

            if context.Publisher() != 'No publisher':
                hdrlist.append( ( 'DC.publisher', context.Publisher() ) )

            created = context.CreationDate()

            #   Filter out DWIMish artifacts on effective / expiration dates
            effective = context.effective_date
            eff_str = ( effective and effective.year() > 1000
                                  and effective != created ) and effective.Date() or ''

            expires = getattr( context, 'expiration_date', None )
            exp_str = ( expires and expires.year() < 9000 ) and expires.Date() or ''

            hdrlist.append( ( 'DC.date.created', created ) )
            hdrlist.append( ( 'DC.date.modified', context.ModificationDate() ) )

            if exp_str or exp_str:
                hdrlist.append( ( 'DC.date.valid_range'
                                , '%s - %s' % ( eff_str, exp_str ) ) )

            hdrlist.append( ( 'DC.type', context.Type() ) )
            hdrlist.append( ( 'DC.format', context.Format() ) )
            hdrlist.append( ( 'DC.language', context.Language() ) )
            hdrlist.append( ( 'DC.rights', context.Rights() ) )

            # Strip empty values
            return filter( lambda x: x[1], hdrlist )



    security.declareProtected(View, 'getRssFeeds')
    def getRssFeeds(self, context):
        """
        Get the feeds availables in the site.
        """

        LOG = logging.getLogger(PROJECTNAME)

        try:
          # We get the configuration.
          rss_feeds_links = self.getProperty('rss_feeds_links', False);
        except Exception, e:
          # We can not get configuration. We fail silently.
          rss_feeds_links = False
          LOG.info("getRssFeeds : Unable to get 'rss_feeds_links' property (Error : %(error)s)." % { 'error' : str(e), })

        try:
            syndication_tool = getToolByName(self, 'portal_syndication')
        except AttributeError, e:
            LOG.error("getRssFeeds : Unable to get 'portal_syndication' (Error : %(error)s)." % { 'error' : str(e), })
            return []

        # We check that syndication is allowed for the site.
        if not syndication_tool.isSiteSyndicationAllowed():
          return []

        if not rss_feeds_links:
            if syndication_tool.isSyndicationAllowed(context):
                rss_item = { 'title' : 'RSS 1.0',
                            'href' : '%s/RSS' % context.absolute_url(),
                           }
                return [ rss_item ]
            else:
                return []

        try:
            catalog_tool = getToolByName(self, 'portal_catalog')
        except AttributeError, e:
            LOG.error("getRssFeeds : Unable to get 'portal_catalog' (Error : %(error)s)." % { 'error' : str(e), })
            return []

        available_topics = catalog_tool.searchResults(
            portal_type='Topic')

        available_feeds = []

        for topic_brain in available_topics:
            rss_item = { 'title' : topic_brain.pretty_title_or_id(),
                         'href' : '%s/RSS' % topic_brain.getURL()
                }
            available_feeds.append(rss_item)

        available_feeds.sort(title_sorter)

        return available_feeds



InitializeClass(EnhancedHeaderTool)
